/* commandstest.c --
   
   Part of: tclmore
   Contents: test commands for the stub table
   Date: Tue Nov 11, 2003
   
   Abstract
   
	This file is the heart of the stub testing library. Here are
	defined function used to test tclmore.
   
   Copyright (c) 2003, 2004 Marco Maggi
   
   The author  hereby grant permission to use,  copy, modify, distribute,
   and  license this  software  and its  documentation  for any  purpose,
   provided that  existing copyright notices  are retained in  all copies
   and that  this notice  is included verbatim  in any  distributions. No
   written agreement, license, or royalty  fee is required for any of the
   authorized uses.  Modifications to this software may be copyrighted by
   their authors and need not  follow the licensing terms described here,
   provided that the new terms are clearly indicated on the first page of
   each file where they apply.
   
   IN NO  EVENT SHALL THE AUTHOR  OR DISTRIBUTORS BE LIABLE  TO ANY PARTY
   FOR  DIRECT, INDIRECT, SPECIAL,  INCIDENTAL, OR  CONSEQUENTIAL DAMAGES
   ARISING OUT  OF THE  USE OF THIS  SOFTWARE, ITS DOCUMENTATION,  OR ANY
   DERIVATIVES  THEREOF, EVEN  IF THE  AUTHOR  HAVE BEEN  ADVISED OF  THE
   POSSIBILITY OF SUCH DAMAGE.
   
   THE  AUTHOR  AND DISTRIBUTORS  SPECIFICALLY  DISCLAIM ANY  WARRANTIES,
   INCLUDING,   BUT   NOT  LIMITED   TO,   THE   IMPLIED  WARRANTIES   OF
   MERCHANTABILITY,    FITNESS   FOR    A    PARTICULAR   PURPOSE,    AND
   NON-INFRINGEMENT.  THIS  SOFTWARE IS PROVIDED  ON AN "AS  IS" BASIS,
   AND  THE  AUTHOR  AND  DISTRIBUTORS  HAVE  NO  OBLIGATION  TO  PROVIDE
   MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
   
*/


/** ------------------------------------------------------------
 ** Header files.
 ** ----------------------------------------------------------*/

#if 0
#  define TCLMORE_DEBUG_MESSAGES	1
#  define TCLMORE_DEBUGGING		1
#endif

#include "tclmoreTest.h"
#include <string.h>
#include <assert.h>


/** ------------------------------------------------------------
 ** Module's global variables.
 ** ----------------------------------------------------------*/

static int		dscriptCounter = 0;
static Tcl_HashTable	dscriptHashTable;
static Tcl_HashTable *	dscriptTable = &dscriptHashTable;


/** ------------------------------------------------------------
 ** Preprocessor symbols.
 ** ----------------------------------------------------------*/

#define debug		More_Debug
#define question	More_Question

#define StateStruct(type)	 type *	state = frame->stateStructPtr


/** ------------------------------------------------------------
 ** Commands from XML document.
 ** ----------------------------------------------------------*/

#include "testcmd.c"



int
OneGreenFunction (More_CommandFrame frame)
{
  Tcl_SetResult(frame->interp, "green", TCL_STATIC);
  return TCL_OK;
}

int
OneWhiteFunction (More_CommandFrame frame)
{
  Tcl_SetResult(frame->interp, "white", TCL_STATIC);
  return TCL_OK;
}

int
OneRedDarkFunction (More_CommandFrame frame)
{
  Tcl_SetResult(frame->interp, "dark", TCL_STATIC);
  return TCL_OK;
}

int
OneRedBrightFunction (More_CommandFrame frame)
{
  Tcl_SetResult(frame->interp, "bright", TCL_STATIC);
  return TCL_OK;
}

int
TwoFunction (More_CommandFrame frame)
{
  Tcl_Obj *	lstObj;
  StateStruct(OneState);


  assert(state->unused1 == 0);
  assert(state->unused2 == 1);
  assert(state->unused3 == 2);
  assert(state->unused4 == 3);
  assert(state->unused5 == 4);
  assert(state->unused6 == 5);

  lstObj = Tcl_NewListObj(0, NULL);
  Tcl_ListObjAppendElement(frame->interp, lstObj,
			   Tcl_NewIntObj(state->integer));
  Tcl_ListObjAppendElement(frame->interp, lstObj,
			   Tcl_NewDoubleObj(state->number));
  Tcl_SetObjResult(frame->interp, lstObj);
  return TCL_OK;
}

int
ThreeFunction (More_CommandFrame frame)
{
  Tcl_Obj *	lstObj;
  StateStruct(TwoState);


  lstObj = Tcl_NewListObj(0, NULL);
  Tcl_ListObjAppendElement(frame->interp, lstObj,
			   Tcl_NewIntObj(state->integer));
  Tcl_ListObjAppendElement(frame->interp, lstObj,
			   Tcl_NewDoubleObj(state->number));
  Tcl_ListObjAppendElement(frame->interp, lstObj,
			   Tcl_NewBooleanObj(state->mode));
  Tcl_SetObjResult(frame->interp, lstObj);
  return TCL_OK;
}

int
FourFunction (More_CommandFrame frame)
{
  Tcl_Obj *	lstObj;
  StateStruct(TwoState);


  lstObj = Tcl_NewListObj(0, NULL);
  Tcl_ListObjAppendElement(frame->interp, lstObj,
			   Tcl_NewIntObj(state->integer));
  Tcl_ListObjAppendElement(frame->interp, lstObj,
			   Tcl_NewDoubleObj(state->number));
  Tcl_ListObjAppendElement(frame->interp, lstObj,
			   Tcl_NewBooleanObj(state->mode));
  Tcl_SetObjResult(frame->interp, lstObj);
  return TCL_OK;
}

int
FivePushFunction (More_CommandFrame frame)
{
  Tcl_SetResult(frame->interp, "push", TCL_STATIC);
  return TCL_OK;
}

int
FivePullMoreFunction (More_CommandFrame frame)
{
  Tcl_Obj *	lstObj;
  StateStruct(TwoState);


  lstObj = Tcl_NewListObj(0, NULL);
  Tcl_ListObjAppendElement(frame->interp, lstObj,
			   Tcl_NewIntObj(state->integer));
  Tcl_ListObjAppendElement(frame->interp, lstObj,
			   Tcl_NewDoubleObj(state->number));
  Tcl_ListObjAppendElement(frame->interp, lstObj,
			   Tcl_NewBooleanObj(state->mode));
  Tcl_SetObjResult(frame->interp, lstObj);
  return TCL_OK;
}

int
FivePullLessFunction (More_CommandFrame frame)
{
  Tcl_Obj *	lstObj;
  StateStruct(TwoState);


  lstObj = Tcl_NewListObj(0, NULL);
  Tcl_ListObjAppendElement(frame->interp, lstObj,
			   Tcl_NewIntObj(state->integer));
  Tcl_ListObjAppendElement(frame->interp, lstObj,
			   Tcl_NewDoubleObj(state->number));
  Tcl_ListObjAppendElement(frame->interp, lstObj,
			   Tcl_NewBooleanObj(state->mode));
  Tcl_SetObjResult(frame->interp, lstObj);
  return TCL_OK;
}

/* BadFunction --

	Callback  for command  used to  test the  inclusion/exclusion of
	unsafe  commands in  safe interpreters.   The command  is marked
	unsafe  in the  XML  document, so  the  TCLMORE function  should
	exclude it from safe interpreters.

   Arguments:

	frame -		Invocation parameters.

   Results:

        Returns  TCL_OK. The  return value  of  the TCL  command is  the
        string "bad".

*/

int
BadFunction (More_CommandFrame frame)
{
  Tcl_SetResult(frame->interp, "bad", TCL_STATIC);
  return TCL_OK;
}

/* SixFunction --

	Callback function  for test command on error  functions. All the
	"More_Error*" functions should be tested here.

   Arguments:

	frame -		Invocation parameters.

   Results:

        Returns TCL_ERROR. The function raises an error.

*/

int
SixFunction (More_CommandFrame frame)
{
  StateStruct(SixState);
  More_Error	error;


  switch (state->index)
    {
    case 0:
      error = More_ErrorNew();
      More_ErrorLogicStr(error, "a logic error");
      return More_ErrorResult(frame->interp, error);

    case 1:
      error = More_ErrorNew();
      More_ErrorLogicStr(error, "a run time error");
      return More_ErrorResult(frame->interp, error);

    case 2:
      error = More_ErrorNew();
      More_ErrorLogicStr(error, "a logic error");
      More_ErrorPrependStr(error, "this is");
      return More_ErrorResult(frame->interp, error);

    case 3:
      error = More_ErrorNew();
      More_ErrorLogicStr(error, "a run time error");
      More_ErrorPrependStr(error, "this is");
      return More_ErrorResult(frame->interp, error);

    case 4:
      error = More_ErrorNew();
      More_ErrorLogicStr(error, "a logic error");
      More_ErrorPrependStr(error, "this is");
      More_ErrorPrependStr(error, "testing");
      return More_ErrorResult(frame->interp, error);

    case 5:
      error = More_ErrorNew();
      More_ErrorLogicStr(error, "a run time error");
      More_ErrorPrependStr(error, "this is");
      More_ErrorPrependStr(error, "testing");
      return More_ErrorResult(frame->interp, error);

    case 6:
      error = More_ErrorNoMemory();
      return More_ErrorResult(frame->interp, error);
    }

  return TCL_OK;
}

/* NewUnsignedFunction --

	Builds a new object holding a "unsigned" value.

   Arguments:

	frame -		Invocation parameters.

   Results:

        Returns TCL_OK or TCL_ERROR.

*/

int
NewUnsignedFunction (More_CommandFrame frame)
{
  StateStruct(ObjState);
  unsigned	value;
  int		e;


  e = More_GetUnsignedFromObj(frame->interp, state->objPtr, &value);
  if (TCL_OK != e)
    {
      Tcl_SetResult(frame->interp, "bad input value", TCL_STATIC);
      return e;
    }
  Tcl_SetObjResult(frame->interp, More_NewUnsignedObj(value));
  return TCL_OK;
}

/* GetUnsignedFunction --

	Callback function for command  that extracts an "unsigned" value
	from an object.

   Arguments:

	frame -		Invocation parameters.

   Results:

        Returns TCL_OK or TCL_ERROR. In case of success the return value
        of the TCL command is the empty string; else an error is raised.

*/

int
GetUnsignedFunction (More_CommandFrame frame)
{
  unsigned	value;
  StateStruct(ObjState);

  return More_GetUnsignedFromObj(frame->interp, state->objPtr, &value);
}

/* GetUnsignedInRangeFunction --

	Callback function for command  that extracts an "unsigned" value
	inside a range, from an object.

   Arguments:

	frame -		Invocation parameters.

   Results:

        Returns TCL_OK or TCL_ERROR. In case of success the return value
        of the TCL command is the empty string; else an error is raised.

*/

int
GetUnsignedInRangeFunction (More_CommandFrame frame)
{
  unsigned	value, min, max;
  StateStruct(ObjState);


  if (More_GetUnsignedFromObj(frame->interp, state->objPtr1, &min) != TCL_OK)
    {
      return TCL_ERROR;
    }
  if (More_GetUnsignedFromObj(frame->interp, state->objPtr2, &max) != TCL_OK)
    {
      return TCL_ERROR;
    }

  return More_GetUnsignedInRangeFromObj(frame->interp, state->objPtr, min, max, &value);
}

/* GetIntInRangeFunction --

	Callback  function  for command  that  extracts  an "int"  value
	inside a range, from an object.

   Arguments:

	frame -		Invocation parameters.

   Results:

        Returns TCL_OK or TCL_ERROR. In case of success the return value
        of the TCL command is the empty string; else an error is raised.

*/

int
GetIntInRangeFunction (More_CommandFrame frame)
{
  int		value, min, max;
  StateStruct(ObjState);


  if (Tcl_GetIntFromObj(frame->interp, state->objPtr1, &min) != TCL_OK)
    {
      return TCL_ERROR;
    }
  if (Tcl_GetIntFromObj(frame->interp, state->objPtr2, &max) != TCL_OK)
    {
      return TCL_ERROR;
    }
  return More_GetIntInRangeFromObj(frame->interp, state->objPtr, min, max, &value);
}

/* GetWideIntInRangeFunction --

	Callback  function for  command that  extracts  an "Tcl_WideInt"
	value inside a range, from an object.

   Arguments:

	frame -		Invocation parameters.

   Results:

        Returns TCL_OK or TCL_ERROR. In case of success the return value
        of the TCL command is the empty string; else an error is raised.

*/

int
GetWideIntInRangeFunction (More_CommandFrame frame)
{
  Tcl_WideInt	value, min, max;
  StateStruct(ObjState);


  if (Tcl_GetWideIntFromObj(frame->interp, state->objPtr1, &min) != TCL_OK)
    {
      return TCL_ERROR;
    }
  if (Tcl_GetWideIntFromObj(frame->interp, state->objPtr2, &max) != TCL_OK)
    {
      return TCL_ERROR;
    }
  return More_GetWideIntInRangeFromObj(frame->interp, state->objPtr, min, max, &value);
}

/* NewSizeTFunction --

	Builds a new object holding a "size_t" value.

   Arguments:

	frame -		Invocation parameters.

   Results:

        Returns TCL_OK or TCL_ERROR.

*/

int
NewSizeTFunction (More_CommandFrame frame)
{
  StateStruct(ObjState);
  size_t	value;
  int		e;


  e = More_GetSizeTFromObj(frame->interp, state->objPtr, &value);
  if (TCL_OK != e)
    {
      Tcl_SetResult(frame->interp, "bad input value", TCL_STATIC);
      return e;
    }
  Tcl_SetObjResult(frame->interp, More_NewSizeTObj(value));
  return TCL_OK;
}

/* GetSizeTFunction --

	Callback  function for  command that  extracts a  "size_t" value
	from an object.

   Arguments:

	frame -		Invocation parameters.

   Results:

        Returns TCL_OK or TCL_ERROR. In case of success the return value
        of the TCL command is the empty string; else an error is raised.

*/

int
GetSizeTFunction (More_CommandFrame frame)
{
  StateStruct(ObjState);
  size_t	value;
  return More_GetSizeTFromObj(frame->interp, state->objPtr, &value);
}

/* GetSizeTInRangeFunction --

	Callback function  for command  that extracts an  "size_t" value
	inside a range, from an object.

   Arguments:

	frame -		Invocation parameters.

   Results:

        Returns TCL_OK or TCL_ERROR. In case of success the return value
        of the TCL command is the empty string; else an error is raised.

*/

int
GetSizeTInRangeFunction (More_CommandFrame frame)
{
  size_t	value, min, max;
  StateStruct(ObjState);


  if (More_GetSizeTFromObj(frame->interp, state->objPtr1, &min) != TCL_OK)
    {
      return TCL_ERROR;
    }
  if (More_GetSizeTFromObj(frame->interp, state->objPtr2, &max) != TCL_OK)
    {
      return TCL_ERROR;
    }
  return More_GetSizeTInRangeFromObj(frame->interp, state->objPtr,
				     min, max, &value);
}

/* NewFloatFunction --

	Builds a new object holding a "float" value.

   Arguments:

	frame -		Invocation parameters.

   Results:

        Returns TCL_OK or TCL_ERROR.

*/

int
NewFloatFunction (More_CommandFrame frame)
{
  StateStruct(ObjState);
  float		value;
  int		e;


  e = More_GetFloatFromObj(frame->interp, state->objPtr, &value);
  if (TCL_OK != e)
    {
      Tcl_SetResult(frame->interp, "bad input value", TCL_STATIC);
      return e;
    }
  Tcl_SetObjResult(frame->interp, More_NewFloatObj(value));
  return TCL_OK;
}

/* GetFloatFunction --

	Callback function for command that extracts a "float" value from
	an object.

   Arguments:

	frame -		Invocation parameters.

   Results:

        Returns TCL_OK or TCL_ERROR. In case of success the return value
        of the TCL command is the empty string; else an error is raised.

*/

int
GetFloatFunction (More_CommandFrame frame)
{
  StateStruct(ObjState);
  float		value;

  return More_GetFloatFromObj(frame->interp, state->objPtr, &value);
}

/* GetFloatInRangeFunction --

	Callback  function for  command  that extracts  a "float"  value
	inside a range, from an object.

   Arguments:

	frame -		Invocation parameters.

   Results:

        Returns TCL_OK or TCL_ERROR. In case of success the return value
        of the TCL command is the empty string; else an error is raised.

*/

int
GetFloatInRangeFunction (More_CommandFrame frame)
{
  float		value, min, max;
  StateStruct(ObjState);


  if (More_GetFloatFromObj(frame->interp, state->objPtr1, &min) != TCL_OK)
    {
      return TCL_ERROR;
    }
  if (More_GetFloatFromObj(frame->interp, state->objPtr2, &max) != TCL_OK)
    {
      return TCL_ERROR;
    }
  return More_GetFloatInRangeFromObj(frame->interp, state->objPtr, min, max, &value);
}

int
MakenameFunction (More_CommandFrame frame)
{
  char *	name;
  StateStruct(ObjState);


  name = More_MakeName(frame->interp, Tcl_GetString(state->objPtr));
  Tcl_SetObjResult(frame->interp, Tcl_NewStringObj(name, -1));
  ckfree(name);
  return TCL_OK;
}

int
UnsafeFunction (More_CommandFrame frame)
{
  Tcl_SetResult(frame->interp, "bad", TCL_STATIC);
  return TCL_OK;
}

int
BufferFunction (More_CommandFrame frame)
{
  StateStruct(BufferState);
  More_Buffer 	buffer;
  More_Block	input;
  More_Block	output;
  int		e;
  size_t	written, read;
  unsigned	i;


  /*
    State structure fields:

    buffer_size -	the requested size of buffers;
    block_size -	number of bytes to read and write;
    times -		how many times to repeat.
  */

  buffer	= More_BufferAlloc();
  More_BufferSetSize(buffer, state->buffer_size);

  input.len	= state->block_size;
  input.ptr	= ckalloc((unsigned) state->block_size);
  output.len	= state->block_size;
  output.ptr	= ckalloc((unsigned) state->block_size);

  for (i=0; i<state->times; ++i)
    {
      debug("writing data to buffer %p", buffer);
      written = 0;
      while (written < state->block_size)
	{
	  More_BufferWrite(buffer, input);
	  written += state->block_size;
	}
      
      debug("reading data from buffer %p", buffer);
      read = 0;
      while ((read < state->block_size) || More_BufferEof(buffer))
	{
	  read = More_BufferRead(buffer, output);
	}

      e = memcmp(input.ptr, output.ptr, (size_t) state->block_size);
      if (e) { break; }
    }
  
  More_BufferFree(buffer);
  ckfree((char *) input.ptr);
  ckfree((char *) output.ptr);

  if (e == 0)
    {
      return TCL_OK;
    }
  else
    {
      Tcl_SetResult(frame->interp, "wrong data", TCL_STATIC);
      return TCL_ERROR;
    }
}

int
AsprintfFunction (More_CommandFrame frame)
{
  StateStruct(AsprintfState);  
  char *	string;

  
  string = More_Asprintf(state->formatString.ptr, state->argumentString.ptr);
  Tcl_SetObjResult(frame->interp, Tcl_NewStringObj(string, -1));
  ckfree(string);
  return TCL_OK;
}
int
ObjprintfFunction (More_CommandFrame frame)
{
  StateStruct(AsprintfState);  

  
  Tcl_SetObjResult(frame->interp,
		   More_ObjPrintf(state->formatString.ptr,
				  state->argumentString.ptr));
  return TCL_OK;
}
int
DsprintfFunction (More_CommandFrame frame)
{
  StateStruct(AsprintfState);  
  Tcl_DString	string;

  
  Tcl_DStringInit(&string);
  Tcl_DStringAppend(&string, "ciao", 4);

  More_DStringPrintf(&string, state->formatString.ptr,
		     state->argumentString.ptr);
  Tcl_SetObjResult(frame->interp,
		   Tcl_NewStringObj(Tcl_DStringValue(&string), -1));
  Tcl_DStringFree(&string);
  return TCL_OK;
}

int
DscriptCreateFunction (More_CommandFrame frame)
{
  DScriptState *	state = (DScriptState *) frame->stateStructPtr;
  char			id[32 + TCL_INTEGER_SPACE];
  Tcl_HashEntry *	entry;
  int			new;
  More_DScript		script;


  if (!dscriptCounter)
    {
      Tcl_InitHashTable(dscriptTable, TCL_STRING_KEYS);
    }
  ++dscriptCounter;
  sprintf(id, "dscript%u", dscriptCounter);
  entry = Tcl_CreateHashEntry(dscriptTable, id, &new);
  if (! new)
    {
      return More_LogicErrorStr(frame->interp, "internal error");
    }

  script = More_DScriptInit(frame->interp, state->script);
  Tcl_SetHashValue(entry, script);
  Tcl_Preserve(script);


  /* try */
  entry  = Tcl_FindHashEntry(dscriptTable, id);
  if (! entry)
    {
      return More_LogicErrorStr(frame->interp, "unknown id");
    }
  script = (More_DScript) Tcl_GetHashValue(entry);
  debug("script \"%s\" token %p", id, script);
  /* end try */

  Tcl_SetObjResult(frame->interp, Tcl_NewStringObj(id, -1));
  return TCL_OK;
}
int
DscriptQueueFunction (More_CommandFrame frame)
{
  DScriptState *	state = (DScriptState *) frame->stateStructPtr;
  Tcl_HashEntry *	entry;
  More_DScript		script, copy;
  More_Objects		objects = MORE_OBJECTS_NULL_VALUE;


  if (!dscriptCounter)
    {
      return More_LogicErrorStr(frame->interp, "no table");
    }

  entry  = Tcl_FindHashEntry(dscriptTable, state->id.ptr);
  script = Tcl_GetHashValue(entry);

  copy = More_DScriptCopy(script);
  if (state->value)
    {
      objects.objv = &(state->value);
      objects.objc = 1;
    }
  More_DScriptQueue(copy, objects);

  return TCL_OK;
}
int
DscriptAbortFunction (More_CommandFrame frame)
{
  DScriptState *	state = (DScriptState *) frame->stateStructPtr;
  Tcl_HashEntry *	entry;
  More_DScript		script;


  debug("aborting script \"%s\"", state->id.ptr);
  if (!dscriptCounter)
    {
      return More_LogicErrorStr(frame->interp, "no table");
    }

  debug("here");
  entry  = Tcl_FindHashEntry(dscriptTable, state->id.ptr);
  if (! entry)
    {
      return More_LogicErrorStr(frame->interp, "unknown id");
    }
  debug("here %p", entry);
  script = (More_DScript) Tcl_GetHashValue(entry);
  debug("releasing script");
  Tcl_Release(script);

  Tcl_DeleteHashEntry(entry);
  return TCL_OK;
}
int
DscriptFinalFunction (More_CommandFrame frame)
{
  Tcl_HashEntry *	entry;
  Tcl_HashSearch	search;


  if (!dscriptCounter)
    {
      return More_LogicErrorStr(frame->interp, "no table");
    }

  debug("finalising hash table");
  for (entry = Tcl_FirstHashEntry(dscriptTable, &search);
       entry;
       entry = Tcl_FirstHashEntry(dscriptTable, &search))
    {
      debug("removing: %s", Tcl_GetHashKey(dscriptTable, entry));
      Tcl_Release(Tcl_GetHashValue(entry));
      Tcl_DeleteHashEntry(entry);
    }
  Tcl_DeleteHashTable(dscriptTable);
  dscriptCounter = 0;
  debug("done");
  return TCL_OK;
}

/* GetModeFromArg --

	Extractor for a switch.

   Arguments:

	frame -		Invocation parameters.

   Results:

        Returns  TCL_OK  if  the  value  is  an  integer,  else  returns
        TCL_ERROR.

*/

int
GetModeFromArg (More_ExtractorFrame frame)
{
  TwoState *	state = frame->commandFrame->stateStructPtr;
  int		mode;
  int *		modePtr;

  /* here we know that it's "-on" or "-off" */
  mode = (strcmp("-on", Tcl_GetString(frame->srcObj)) == 0)? 1 : 0;
  if (frame->fieldPtr)
    {
      modePtr = (int *) frame->fieldPtr;
      *modePtr = mode;
    }
  else
    {
      state->mode = mode;
    }
  return TCL_OK;
}

/* GetSixModeFromArg --

	Extracts the argument for the [six] command.

   Arguments:

	args -		Invocation parameters.
	argObj -	Pointer to the source object.
	offset -	Offset of the integer in the structure.
	D -		Pointer to the invocation state structure.

   Results:

        Returns  TCL_OK  if  the  value  is  an  integer,  else  returns
        TCL_ERROR.

*/

int
GetSixModeFromArg (More_ExtractorFrame frame)
{
  static CONST char * modes[] = {
    "logic", "runtime",
    "more logic", "more runtime",
    "even more logic", "even more runtime",
    "no memory",
    "nothing", NULL
  };
  int	e, mode;
  int * modePtr;


  e = Tcl_GetIndexFromObj(frame->commandFrame->interp, frame->srcObj,
			  modes, "mode", TCL_EXACT, &mode);
  if (TCL_OK != e)
    {
      return e;
    }
  modePtr = (int *) frame->fieldPtr;
  *modePtr = mode;
  return TCL_OK;
}


int
GetObjFromArg (More_ExtractorFrame frame)
{
  return More_GetObjFromArg(frame);
}
int
GetIntFromArg (More_ExtractorFrame frame)
{
  return More_GetIntFromArg(frame);
}
int
GetUnsignedFromArg (More_ExtractorFrame frame)
{
  return More_GetUnsignedFromArg(frame);
}
int
GetFloatFromArg (More_ExtractorFrame frame)
{
  return More_GetFloatFromArg(frame);
}
int
GetDoubleFromArg (More_ExtractorFrame frame)
{
  return More_GetDoubleFromArg(frame);
}
int
GetSizeTFromArg (More_ExtractorFrame frame)
{
  return More_GetSizeTFromArg(frame);
}
int
GetAStringFromArg (More_ExtractorFrame frame)
{
  return More_GetAStringFromArg(frame);
}


/* end of file */
/*
Local Variables:
mode: c
page-delimiter: "^$"
End:
*/
