/* dstring.c
   
   Part of: tclmore
   Contents: dynamic string functions
   Date: Fri Feb 27, 2004
   
   Abstract
   
   
   
   Copyright (c) 2004 Marco Maggi
   
   The author  hereby grant permission to use,  copy, modify, distribute,
   and  license this  software  and its  documentation  for any  purpose,
   provided that  existing copyright notices  are retained in  all copies
   and that  this notice  is included verbatim  in any  distributions. No
   written agreement, license, or royalty  fee is required for any of the
   authorized uses.  Modifications to this software may be copyrighted by
   their authors and need not  follow the licensing terms described here,
   provided that the new terms are clearly indicated on the first page of
   each file where they apply.
   
   IN NO  EVENT SHALL THE AUTHOR  OR DISTRIBUTORS BE LIABLE  TO ANY PARTY
   FOR  DIRECT, INDIRECT, SPECIAL,  INCIDENTAL, OR  CONSEQUENTIAL DAMAGES
   ARISING OUT  OF THE  USE OF THIS  SOFTWARE, ITS DOCUMENTATION,  OR ANY
   DERIVATIVES  THEREOF, EVEN  IF THE  AUTHOR  HAVE BEEN  ADVISED OF  THE
   POSSIBILITY OF SUCH DAMAGE.
   
   THE  AUTHOR  AND DISTRIBUTORS  SPECIFICALLY  DISCLAIM ANY  WARRANTIES,
   INCLUDING,   BUT   NOT  LIMITED   TO,   THE   IMPLIED  WARRANTIES   OF
   MERCHANTABILITY,    FITNESS   FOR    A    PARTICULAR   PURPOSE,    AND
   NON-INFRINGEMENT.  THIS  SOFTWARE IS PROVIDED  ON AN "AS  IS" BASIS,
   AND  THE  AUTHOR  AND  DISTRIBUTORS  HAVE  NO  OBLIGATION  TO  PROVIDE
   MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
   
*/


/** ------------------------------------------------------------
 ** Header files.
 ** ----------------------------------------------------------*/

#if 1
#  define TCLMORE_DEBUG_MESSAGES
#  define TCLMORE_DEBUGGING		1
#endif

#include <assert.h>
#include <limits.h>
#include <stddef.h>
#include <float.h>
#include "tclmoreInt.h"


/** ------------------------------------------------------------
 ** Preprocessor symbols.
 ** ----------------------------------------------------------*/

#define TCLMORE_FLOAT_MAX	FLT_MAX
#define TCLMORE_FLOAT_MIN	FLT_MIN

#define debug			More_Debug


/* More_DStringPrintf --

	A printf() version that appends the result to a dynamic string.

   Arguments:

	string -	Pointer to the dynamic string.
	format -	Pointer to the format string.
	... -		Arguments.

   Results:

        Formats the string appending the result to the dynamic string.

*/

void
More_DStringPrintf(Tcl_DString *string, CONST char *format, ...)
{
  char *	p;
  va_list	ap;


  va_start(ap, format);
  p = More_Bsprintf(format, ap);
  Tcl_DStringAppend(string, p, -1);
  ckfree(p);
}

/* More_DStringAppendSizeT --

	Appends a "size_t" value to a dynamic string.

   Arguments:

	string -	Pointer to the string.
	size -		The value.

   Results:

        Formats the string.

*/

void
More_DStringAppendSizeT (Tcl_DString *string, size_t size)
{
  More_DStringPrintf(string, TCLMORE_SIZE_T_PRINTF, size);
}


/* end of file */
/*
Local Variables:
mode: c
page-delimiter: "^$"
End:
*/
