/* init.c --
   
   Part of: tclmore
   Contents: library initialisation functions
   Date: Fri Apr 26, 2002
   
   Abstract
   
	Library initialisation.   
   
   Copyright (c) 2002, 2003, 2004 Marco Maggi
   
   The author  hereby grant permission to use,  copy, modify, distribute,
   and  license this  software  and its  documentation  for any  purpose,
   provided that  existing copyright notices  are retained in  all copies
   and that  this notice  is included verbatim  in any  distributions. No
   written agreement, license, or royalty  fee is required for any of the
   authorized uses.  Modifications to this software may be copyrighted by
   their authors and need not  follow the licensing terms described here,
   provided that the new terms are clearly indicated on the first page of
   each file where they apply.
   
   IN NO  EVENT SHALL THE AUTHOR  OR DISTRIBUTORS BE LIABLE  TO ANY PARTY
   FOR  DIRECT, INDIRECT, SPECIAL,  INCIDENTAL, OR  CONSEQUENTIAL DAMAGES
   ARISING OUT  OF THE  USE OF THIS  SOFTWARE, ITS DOCUMENTATION,  OR ANY
   DERIVATIVES  THEREOF, EVEN  IF THE  AUTHOR  HAVE BEEN  ADVISED OF  THE
   POSSIBILITY OF SUCH DAMAGE.
   
   THE  AUTHOR  AND DISTRIBUTORS  SPECIFICALLY  DISCLAIM ANY  WARRANTIES,
   INCLUDING,   BUT   NOT  LIMITED   TO,   THE   IMPLIED  WARRANTIES   OF
   MERCHANTABILITY,    FITNESS   FOR    A    PARTICULAR   PURPOSE,    AND
   NON-INFRINGEMENT.  THIS  SOFTWARE IS PROVIDED  ON AN "AS  IS" BASIS,
   AND  THE  AUTHOR  AND  DISTRIBUTORS  HAVE  NO  OBLIGATION  TO  PROVIDE
   MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
*/


/** ------------------------------------------------------------
 ** Header files.
 ** ----------------------------------------------------------*/

#include "tclmoreInt.h"


static Tcl_InterpDeleteProc DeleteAssocData;


/* Tclmore_Init --

	Package initialisation.

   Arguments:

	interp: the interp in which register the package

   Results:

        Returns TCL_OK or TCL_ERROR.

   Side effects:

        None.

*/

EXTERN int
Tclmore_Init (Tcl_Interp *interp)
{
  int			e;
  More_AssocData *	data;
  More_Error		error;
  

#ifdef USE_TCL_STUBS
#warning using TCL stubs
  if (Tcl_InitStubs(interp, "8.4", 0) == NULL) {
    return TCL_ERROR;
  }
#endif


  e = Tcl_PkgProvideEx(interp,
		       TCL_QUOTED_PACKAGE_NAME,
		       TCL_QUOTED_PACKAGE_VERSION,
		       (ClientData) &moreStub);
  if (e == TCL_OK)
    {
      data = (More_AssocData *) ckalloc(sizeof(More_AssocData));
      Tcl_SetAssocData(interp, More_AssocKey, DeleteAssocData, data);
      data->channelCountersInitialised = 0;

      error = More_CreateCommands(interp, NULL, More_Commands);
      if (error)
	{
	  return More_ErrorResult(interp, error);
	}
    }
  
  return e;
}

/* Tclmore_SafeInit --

	Initialise the package in a safe interp.

   Arguments:

	interp: the interp in which register the package

   Results:

        Returns TCL_OK or TCL_ERROR.

   Side effects:

        None.

*/

EXTERN int
Tclmore_SafeInit(Tcl_Interp *interp)
{
  return Tclmore_Init(interp);
}

/* DeleteAssocData --

	Deletes the assoc data for an interpreter.

   Arguments:

	D -		Pointer to the assoc data structure.
	interp -	The interpreter.

   Results:

        Frees the structure.

*/

void
DeleteAssocData (ClientData D, Tcl_Interp *interp)
{
  More_AssocData	* data = D;

  if (data->channelCountersInitialised)
    {
      Tcl_DeleteHashTable(&(data->channelCounters));
    }
  ckfree((char *) D);
}

/* end of file */
/*
Local Variables:
mode: c
End:
*/
