/* pipechan.c --
   
   Part of: tclmore
   Contents: coupled channels interface
   Date: Fri Jan 23, 2004
   
   Abstract
   
	This  module implements  "pipe channels":  a couple  of channels
	linked  together  through shared  buffers.  The  channels are  a
	couple of buffer channels.
   
   Copyright (c) 2004 Marco Maggi
   
   The author  hereby grant permission to use,  copy, modify, distribute,
   and  license this  software  and its  documentation  for any  purpose,
   provided that  existing copyright notices  are retained in  all copies
   and that  this notice  is included verbatim  in any  distributions. No
   written agreement, license, or royalty  fee is required for any of the
   authorized uses.  Modifications to this software may be copyrighted by
   their authors and need not  follow the licensing terms described here,
   provided that the new terms are clearly indicated on the first page of
   each file where they apply.
   
   IN NO  EVENT SHALL THE AUTHOR  OR DISTRIBUTORS BE LIABLE  TO ANY PARTY
   FOR  DIRECT, INDIRECT, SPECIAL,  INCIDENTAL, OR  CONSEQUENTIAL DAMAGES
   ARISING OUT  OF THE  USE OF THIS  SOFTWARE, ITS DOCUMENTATION,  OR ANY
   DERIVATIVES  THEREOF, EVEN  IF THE  AUTHOR  HAVE BEEN  ADVISED OF  THE
   POSSIBILITY OF SUCH DAMAGE.
   
   THE  AUTHOR  AND DISTRIBUTORS  SPECIFICALLY  DISCLAIM ANY  WARRANTIES,
   INCLUDING,   BUT   NOT  LIMITED   TO,   THE   IMPLIED  WARRANTIES   OF
   MERCHANTABILITY,    FITNESS   FOR    A    PARTICULAR   PURPOSE,    AND
   NON-INFRINGEMENT.  THIS  SOFTWARE IS PROVIDED  ON AN "AS  IS" BASIS,
   AND  THE  AUTHOR  AND  DISTRIBUTORS  HAVE  NO  OBLIGATION  TO  PROVIDE
   MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
*/


/** ------------------------------------------------------------
 ** Header files.
 ** ----------------------------------------------------------*/

#if 0
#  define TCLMORE_DEBUG_MESSAGES
#  define TCLMORE_DEBUGGING		1
#endif

#include <string.h>
#include <errno.h>
#include <assert.h>
#include <stddef.h>
#include "tclmoreInt.h"


/** ------------------------------------------------------------
 ** Preprocessor symbols.
 ** ----------------------------------------------------------*/

#define debug			More_Debug
#define question(expr)		More_Question


/* More_OpenPipeChannel --

	Opens a  new couple of channels, linked  together through shared
	buffers.   If mode is  TCL_READABLE: the  first channel  will be
	read-only   and  the   second   one  write-only;   if  mode   is
	TCL_WRITABLE: the opposite.  If  mode is TCL_READABLE OR-ed with
	TCL_WRITABLE both the channels will be read-write.

   Arguments:

	modeMask -	An OR-ed combination of TCL_READABLE and
			TCL_WRITABLE. It must not be zero.
	channelVar -	Pointer to the variable that will hold the
			second channel token.

   Results:

	Returns the first channel token.

*/

Tcl_Channel
More_OpenPipeChannel (int modeMask, Tcl_Channel *channelVar)
{
  More_Buffer	inputBuffer  = NULL;
  More_Buffer	outputBuffer = NULL;
  Tcl_Channel	channelOne;
  Tcl_Channel	channelTwo;


  debug("creating pipe channel");

  if (modeMask & TCL_READABLE)
    {
      inputBuffer  = More_BufferAlloc();
    }
  if (modeMask & TCL_WRITABLE)
    {
      outputBuffer = More_BufferAlloc();
    }


  channelOne = More_OpenBufferChannel(inputBuffer, outputBuffer);
  channelTwo = More_OpenBufferChannel(outputBuffer, inputBuffer);

  debug("created channel one: \"%s\", thread %p",
	Tcl_GetChannelName(channelOne), Tcl_GetChannelThread(channelOne));
  debug("created channel two: \"%s\", thread %p",
	Tcl_GetChannelName(channelTwo), Tcl_GetChannelThread(channelTwo));

  *channelVar = channelTwo;
  return channelOne;
}


/* end of file */
/*
Local Variables:
mode: c
page-delimiter: "^$"
End:
*/
