/* tclmoreStubInit.c --
   
   Part of: tclmore
   Contents: stub table
   Date: Wed Nov 12, 2003
   
   Abstract
   
	This file defines the stub  table: an array of function pointers
	used to  allow external  dynamically linked libraries  to access
	functions in this extension.
   
   Copyright (c) 2003, 2004 Marco Maggi
   
   The author hereby grant  permission to use, copy, modify, distribute,
   and  license this  software and  its documentation  for  any purpose,
   provided that  existing copyright notices are retained  in all copies
   and that  this notice is  included verbatim in any  distributions. No
   written agreement, license, or royalty fee is required for any of the
   authorized uses.   Modifications to this software  may be copyrighted
   by their  authors and need  not follow the licensing  terms described
   here, provided that the new  terms are clearly indicated on the first
   page of each file where they apply.
   
   IN NO EVENT  SHALL THE AUTHOR OR DISTRIBUTORS BE  LIABLE TO ANY PARTY
   FOR DIRECT,  INDIRECT, SPECIAL, INCIDENTAL,  OR CONSEQUENTIAL DAMAGES
   ARISING OUT  OF THE USE OF  THIS SOFTWARE, ITS  DOCUMENTATION, OR ANY
   DERIVATIVES  THEREOF, EVEN  IF THE  AUTHOR HAVE  BEEN ADVISED  OF THE
   POSSIBILITY OF SUCH DAMAGE.
   
   THE  AUTHOR AND  DISTRIBUTORS SPECIFICALLY  DISCLAIM  ANY WARRANTIES,
   INCLUDING,   BUT  NOT   LIMITED   TO,  THE   IMPLIED  WARRANTIES   OF
   MERCHANTABILITY,    FITNESS   FOR    A   PARTICULAR    PURPOSE,   AND
   NON-INFRINGEMENT.  THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, AND
   THE   AUTHOR  AND   DISTRIBUTORS  HAVE   NO  OBLIGATION   TO  PROVIDE
   MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
   
*/

#include "tclmoreInt.h"


/** ------------------------------------------------------------
 ** Stub table for error functions.
 ** ----------------------------------------------------------*/

CONST More_StubTableError moreStubError = {
  More_ErrorNew,
  More_ErrorForget,
  More_ErrorResult,
  More_ErrorLogic,
  More_ErrorRuntime,
  More_ErrorLogicStr,
  More_ErrorRuntimeStr,
  More_ErrorPrepend,
  More_ErrorPrependStr,
  More_ErrorIsLogic,
  More_ErrorIsRuntime,
  More_ErrorSetLogic,
  More_ErrorSetRuntime,

  More_ErrorNoMemory,
  More_ErrorErrno,
  More_ErrorCodeAndForget,

  More_LogicError,
  More_LogicErrorStr,
  More_RuntimeError,
  More_RuntimeErrorStr,
  More_WrongNumArgs,
  More_OptionRequiresArg
};


/** ------------------------------------------------------------
 ** Stub table for id table functions.
 ** ----------------------------------------------------------*/

CONST More_StubTableIdTable moreStubIdTable = {
  More_InitIdTable,
  More_DeleteIdTable,
  More_AttachId,
  More_DetachId,
  More_GetDataFromId
};


/** ------------------------------------------------------------
 ** Stub table for TCL variable functions.
 ** ----------------------------------------------------------*/

CONST More_StubTableVariable moreStubVariable = {
  More_VariableInit,
  More_VariableCopy,
  More_VariableFinal,
  More_VariableTest,
  More_VariableExists,
  More_VariableSet,
  More_VariableSetBlock,
  More_VariableAppend,
  More_VariableAppendBlock,
  More_VariableClear,
  More_VariableGet,
  More_VariableGetBlock,
  More_VariableName
};


/** ------------------------------------------------------------
 ** Stub table for channel interface functions.
 ** ----------------------------------------------------------*/

CONST More_StubTableChannelInterface moreStubChannelInterface = {
  More_CreateBufferVariable,
  More_DeleteBufferVariable,
  More_OpenBufferChannel,
  More_OpenVarChannel,
  More_StackTeeChannel,
  More_OpenPipeChannel,
  More_GetOpenModeFromObj,
  More_GetTransformOpenModeFromObj,
  More_GetChannelFromObj
};


/** ------------------------------------------------------------
 ** Stub table for channel driver functions.
 ** ----------------------------------------------------------*/

CONST More_StubTableChannelDriver moreStubChannelDriver = {
  More_ChannelDriverSetOption,
  More_ChannelDriverGetOption
};


/** ------------------------------------------------------------
 ** Stub table for object extractor functions.
 ** ----------------------------------------------------------*/

CONST More_StubTableObjectExtractor moreStubObjectExtractor = {
  More_NewUnsignedObj,
  More_GetUnsignedFromObj,
  More_GetUnsignedInRangeFromObj,
  More_GetIntInRangeFromObj,
  More_GetWideIntInRangeFromObj,
  More_NewSizeTObj,
  More_GetSizeTFromObj,
  More_GetSizeTInRangeFromObj,
  More_NewFloatObj,
  More_GetFloatFromObj,
  More_GetFloatInRangeFromObj
};


/** ------------------------------------------------------------
 ** Stub table for argument extractor functions.
 ** ----------------------------------------------------------*/

CONST More_StubTableArgumentExtractor moreStubArgumentExtractor = {
  More_GetObjFromArg,
  More_GetStringFromArg,
  More_GetAStringFromArg,
  More_GetBlockFromArg,
  More_GetABlockFromArg,
  More_GetIntFromArg,
  More_GetWideIntFromArg,
  More_GetUnsignedFromArg,
  More_GetFloatFromArg,
  More_GetDoubleFromArg,
  More_GetSizeTFromArg
};


/** ------------------------------------------------------------
 ** Stub table for buffer functions.
 ** ----------------------------------------------------------*/

CONST More_StubTableBuffer moreStubBuffer = {
  More_BufferAlloc,
  More_BufferFree,
  More_BufferRead,
  More_BufferReadAllBlock,
  More_BufferReadAllObj,
  More_BufferWrite,
  More_BufferEmpty,
  More_BufferEof,
  More_BufferAlive,
  More_BufferReaderCallback,
  More_BufferWriterCallback,
  More_BufferDetachReader,
  More_BufferDetachWriter,
  More_BufferSetSize,
  More_BufferGetSize
};


/** ------------------------------------------------------------
 ** Stub table for dynamic string functions.
 ** ----------------------------------------------------------*/

CONST More_StubTableDString moreStubDString = {
  More_DStringPrintf,
  More_DStringAppendSizeT
};


/** ------------------------------------------------------------
 ** Stub table for stream functions.
 ** ----------------------------------------------------------*/

CONST More_StubTableStream moreStubStream = {
  More_MakeStreamTransform
};


/** ------------------------------------------------------------
 ** Stub table for miscellaneous functions.
 ** ----------------------------------------------------------*/

CONST More_StubTableMisc moreStubMisc = {
  More_EqualVarnames,
  More_MakeName,
  More_DupAllocString,
  More_Asprintf,
  More_Bsprintf,
  More_ObjPrintf
};


/** ------------------------------------------------------------
 ** Stub table for delayed scripts.
 ** ----------------------------------------------------------*/

CONST More_StubTableDelayedScript moreStubDelayedScript = {
  More_DScriptInit,
  More_DScriptCopy,
  More_DScriptFinal,
  More_DScriptQueue,
  More_DScriptCallback
};


/** ------------------------------------------------------------
 ** Global stub table.
 ** ----------------------------------------------------------*/

CONST More_StubTable moreStub = {

  &moreStubError,
  &moreStubIdTable,
  &moreStubVariable,
  &moreStubChannelInterface,
  &moreStubChannelDriver,
  &moreStubObjectExtractor,
  &moreStubArgumentExtractor,
  &moreStubBuffer,
  &moreStubDString,
  &moreStubStream,
  &moreStubMisc,
  &moreStubDelayedScript,

  More_CreateCommands
};


/* end of file */
