/* varchan.c --
   
   Part of: tclmore
   Contents: channel interface to the value stored in a TCL variable
   Date: Thu Jan  1, 2004
   
   Abstract
   
	A  varchan is  a  channel interface  to  the contents  of a  TCL
	variable. It's  purpose is to serve  as source of  data or final
	data collector for stacked channels, to allow easier debug.
   
   Copyright (c) 2004 Marco Maggi
   
   The author  hereby grant permission to use,  copy, modify, distribute,
   and  license this  software  and its  documentation  for any  purpose,
   provided that  existing copyright notices  are retained in  all copies
   and that  this notice  is included verbatim  in any  distributions. No
   written agreement, license, or royalty  fee is required for any of the
   authorized uses.  Modifications to this software may be copyrighted by
   their authors and need not  follow the licensing terms described here,
   provided that the new terms are clearly indicated on the first page of
   each file where they apply.
   
   IN NO  EVENT SHALL THE AUTHOR  OR DISTRIBUTORS BE LIABLE  TO ANY PARTY
   FOR  DIRECT, INDIRECT, SPECIAL,  INCIDENTAL, OR  CONSEQUENTIAL DAMAGES
   ARISING OUT  OF THE  USE OF THIS  SOFTWARE, ITS DOCUMENTATION,  OR ANY
   DERIVATIVES  THEREOF, EVEN  IF THE  AUTHOR  HAVE BEEN  ADVISED OF  THE
   POSSIBILITY OF SUCH DAMAGE.
   
   THE  AUTHOR  AND DISTRIBUTORS  SPECIFICALLY  DISCLAIM ANY  WARRANTIES,
   INCLUDING,   BUT   NOT  LIMITED   TO,   THE   IMPLIED  WARRANTIES   OF
   MERCHANTABILITY,    FITNESS   FOR    A    PARTICULAR   PURPOSE,    AND
   NON-INFRINGEMENT.  THIS  SOFTWARE IS PROVIDED  ON AN "AS  IS" BASIS,
   AND  THE  AUTHOR  AND  DISTRIBUTORS  HAVE  NO  OBLIGATION  TO  PROVIDE
   MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
   
*/


/** ------------------------------------------------------------
 ** Header files.
 ** ----------------------------------------------------------*/

#if 0
#  define TCLMORE_DEBUG_MESSAGES
#  define TCLMORE_DEBUGGING		1
#endif 

#include <string.h>
#include <errno.h>
#include <assert.h>
#include <stddef.h>
#include "tclmoreInt.h"


/** ------------------------------------------------------------
 ** Preprocessor symbols.
 ** ----------------------------------------------------------*/

#define debug			More_Debug
#define question		More_Question


/* More_OpenVarChannel --

	Opens a new channel interface for the contents of a variable. If
	TCL_READABLE  is used:  bytes stored  in the  variable  are read
	later from  the channel; if TCL_WRITABLE is  used: bytes written
	into the channel are found later in the variable.

   Arguments:

	variable -	Variable data.
	modeMask -	An OR-ed combination of TCL_READABLE and
			TCL_WRITABLE.

   Results:

	Returns the channel token.

*/

Tcl_Channel
More_OpenVarChannel (More_Variable variable, int modeMask)
{
  /*
    "input" and "output" from the point of view of the variable.
   */
  More_Buffer	inputBuffer  = NULL;
  More_Buffer	outputBuffer = NULL;


  debug("channel for variable \"%s\"", Tcl_GetString(variable[0].name));

  if (modeMask & TCL_WRITABLE)
    {
      debug("allocating input buffer");
      inputBuffer  = More_BufferAlloc();
    }
  if (modeMask & TCL_READABLE)
    {
      debug("allocating output buffer");
      outputBuffer = More_BufferAlloc();
    }
  
  More_CreateBufferVariable(variable, inputBuffer, outputBuffer);
  return More_OpenBufferChannel(outputBuffer, inputBuffer);
}

/* end of file */
/*
Local Variables:
mode: c
page-delimiter: "^$"
End:
*/
